@@ -53,6 +53,7 @@ demandimport.enable()
 from mercurial import (
     error,
     pycompat,
+    util,
 )
 
 from mercurial.windows import posixfile, unlink, rename
@@ -287,7 +288,7 @@ def update_batch(batch):
         # TODO: Figure out if this hack for py2exe is still needed, and possibly
         #       wrap this to handle unicode on py3
         if not pycompat.ispy3:
-            errorstream = pycompat.bytesio()
+            errorstream = util.bytesio()
             sys.stderr = errorstream
         try:
             # Ensure that all unset dirstate entries can be updated.
