@@ -14,15 +14,15 @@ import pathlib
 import shutil
 import subprocess
 
-from hgpackaging.downloads import (
+from .downloads import (
     DOWNLOADS,
     download_entry,
 )
-from hgpackaging import (
-    util as hgutil,
+from . import (
+    util as thgutil,
     wix,
 )
-from hgpackaging.util import (
+from .util import (
     extract_tar_to_directory,
     extract_zip_to_directory,
     process_install_rules,
@@ -87,7 +87,7 @@ def build_py2(
 
     venv_bin, venv_pip, venv_python = _make_venv(python_exe, build_dir, output)
 
-    py_info = hgutil.python_exe_info(python_exe)
+    py_info = thgutil.python_exe_info(python_exe)
     vc_x64 = py_info['arch'] == '64bit'
 
     if os.name == 'nt':
