@@ -160,9 +160,9 @@ class GcInfoDialog(QDialog):
 
         self._buttonBox = bbox = QDialogButtonBox(self)
         self.layout().addWidget(bbox)
-        b = bbox.addButton('&Show Widget', QDialogButtonBox.ActionRole)
+        b = bbox.addButton('&Show Widget', QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.showWidget)
-        b = bbox.addButton('&Destroy', QDialogButtonBox.ResetRole)
+        b = bbox.addButton('&Destroy', QDialogButtonBox.ButtonRole.ResetRole)
         b.clicked.connect(self.deleteWidget)
         b.setAutoDefault(False)
 
