@@ -5,12 +5,9 @@ from __future__ import absolute_import
 import os
 import re
 import sys
-import tempfile
 import time
 from collections import defaultdict
 
-from nose import tools
-
 from tortoisehg.hgqt.qtcore import (
     QTextCodec,
 )
@@ -20,6 +17,7 @@ from mercurial import (
     encoding as encodingmod,
     error,
     pycompat,
+    util,
 )
 
 from tortoisehg.util import hglib
@@ -101,7 +99,16 @@ def patchencoding(encoding, fallbackencoding=None):
 def with_encoding(encoding, fallbackencoding=None):
     """Decorator for test function to change locale encoding temporarily"""
     patcher = EncodingPatcher(encoding, fallbackencoding)
-    return tools.with_setup(patcher.patch, patcher.restore)
+    def deco(f):
+        def wrapper():
+            patcher.patch()
+            try:
+                f()
+            finally:
+                patcher.restore()
+        wrapper.__name__ = f.__name__
+        return wrapper
+    return deco
 
 class HgClient(object):
     """Mercurial client to set up fixture repository
@@ -152,8 +159,8 @@ class HgClient(object):
         origwd = os.getcwd()
         ui = hglib.loadui()
         ui.setconfig(b'ui', b'strict', True)
-        ui.fout = pycompat.bytesio()
-        ui.ferr = pycompat.bytesio()
+        ui.fout = util.bytesio()
+        ui.ferr = util.bytesio()
         args = list(args)
         req = dispatch.request(args, ui=ui)
         options = dispatch._earlyparseopts(ui, args)
