@@ -120,8 +120,8 @@ class HgEnvPlugin(object):
         # settings will be saved at $HGTMP/.config/TortoiseHg/TortoiseHgQt.ini
         self._qapp.setApplicationName('TortoiseHgQt')
         self._qapp.setOrganizationName('TortoiseHg')
-        QSettings.setDefaultFormat(QSettings.IniFormat)
-        QSettings.setPath(QSettings.IniFormat, QSettings.UserScope,
+        QSettings.setDefaultFormat(QSettings.Format.IniFormat)
+        QSettings.setPath(QSettings.Format.IniFormat, QSettings.Scope.UserScope,
                           self.configdir)
 
     def pytest_sessionfinish(self):
