@@ -9,8 +9,6 @@ from mercurial import (
     pycompat,
 )
 
-from nose.plugins.skip import SkipTest
-
 import pytest
 
 from tortoisehg.hgqt.qtcore import (
@@ -32,7 +30,7 @@ import helpers
 def setup_module():
     # necessary for style().standardIcon()
     if not isinstance(QApplication.instance(), QApplication):
-        raise SkipTest
+        raise unittest.SkipTest
 
     global _tmpdir
     _tmpdir = helpers.mktmpdir(__name__)
@@ -161,8 +159,8 @@ class ManifestModelTest(unittest.TestCase):
     def test_drag_flag(self):
         m = self.new_model(0)
         # only files are drag enabled
-        self.assertTrue(m.flags(m.indexFromPath('foo')) & Qt.ItemIsDragEnabled)
-        self.assertFalse(m.flags(m.indexFromPath('baz')) & Qt.ItemIsDragEnabled)
+        self.assertTrue(m.flags(m.indexFromPath('foo')) & Qt.ItemFlag.ItemIsDragEnabled)
+        self.assertFalse(m.flags(m.indexFromPath('baz')) & Qt.ItemFlag.ItemIsDragEnabled)
 
     def test_isdir(self):
         m = self.new_model(3)
@@ -457,7 +455,7 @@ class ManifestModelTest(unittest.TestCase):
         m.setRawContext(repo[1])
         self.assertEqual(1, m.rev())
         # should not fall back to patch handler
-        self.assertTrue(m.flags(m.indexFromPath('foo')) & Qt.ItemIsDragEnabled)
+        self.assertTrue(m.flags(m.indexFromPath('foo')) & Qt.ItemFlag.ItemIsDragEnabled)
 
     def test_change_base_rev(self):
         m = self.new_model(10)
@@ -537,7 +535,7 @@ class ManifestModelPatchTest(unittest.TestCase):
 
     def test_no_drag_flag(self):
         m = self.new_model(b'patch0.diff')
-        self.assertFalse(m.flags(m.indexFromPath('foo')) & Qt.ItemIsDragEnabled)
+        self.assertFalse(m.flags(m.indexFromPath('foo')) & Qt.ItemFlag.ItemIsDragEnabled)
 
     def test_status(self):
         m = self.new_model(b'patch0.diff')
