@@ -3,7 +3,7 @@
 # Copyright 2010 Yuki KODAMA <endflow.net@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -186,7 +186,7 @@ class SummaryInfo(object):
             elif item == 'branch':
                 value = self.get_data('rawbranch', *args)
                 if value:
-                    repo = ctx._repo
+                    repo = ctx.repo()
                     try:
                         if ctx.node() != repo.branchtip(ctx.branch()):
                             return None
@@ -260,7 +260,7 @@ class SummaryInfo(object):
                 p1 = ctx.p1()
                 if p1 is not None and p1.hex() == target:
                     return None
-                if target not in ctx._repo:
+                if target not in ctx.repo():
                     return None
                 return hglib.tounicode(target)
             raise UnknownItem(item)
@@ -448,23 +448,23 @@ class SummaryPanel(SummaryBase, QWidget):
         if self.revlabel is None:
             self.revlabel = QLabel()
             self.revlabel.linkActivated.connect(self.linkActivated)
-            layout.addWidget(self.revlabel, 0, Qt.AlignTop)
+            layout.addWidget(self.revlabel, 0, Qt.AlignmentFlag.AlignTop)
 
         if 'expandable' in self.csstyle and self.csstyle['expandable']:
             if self.expand_btn.parentWidget() is None:
                 self.expand_btn.clicked.connect(lambda: self.update())
                 margin = QHBoxLayout()
                 margin.setContentsMargins(3, 3, 3, 3)
-                margin.addWidget(self.expand_btn, 0, Qt.AlignTop)
+                margin.addWidget(self.expand_btn, 0, Qt.AlignmentFlag.AlignTop)
                 layout.insertLayout(0, margin)
             self.expand_btn.setVisible(True)
         elif self.expand_btn.parentWidget() is not None:
             self.expand_btn.setHidden(True)
 
-        interact = Qt.LinksAccessibleByMouse
+        interact = Qt.TextInteractionFlag.LinksAccessibleByMouse
 
         if 'selectable' in self.csstyle and self.csstyle['selectable']:
-            interact |= Qt.TextBrowserInteraction
+            interact |= Qt.TextInteractionFlag.TextBrowserInteraction
 
         self.revlabel.setTextInteractionFlags(interact)
 
@@ -525,7 +525,7 @@ class SummaryLabel(SummaryBase, QLabel):
 
         if 'selectable' in self.csstyle:
             sel = self.csstyle['selectable']
-            val = sel and Qt.TextSelectableByMouse or Qt.TextBrowserInteraction
+            val = sel and Qt.TextInteractionFlag.TextSelectableByMouse or Qt.TextInteractionFlag.TextBrowserInteraction
             self.setTextInteractionFlags(val)
 
         if 'width' in self.csstyle:
