@@ -53,6 +53,27 @@ from . import (
     graphopt,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        FrozenSet,
+        List,
+        Optional,
+        Text,
+        Tuple,
+        Union,
+    )
+    from mercurial import context
+    from .qtcore import QObject
+    from . import (
+        thgrepo,
+        filedialogs,
+    )
+
+    # _UnfilteredRepoAgentProxy is a short term hack, so don't use this type
+    # for RepoAgent everywhere.
+    RepoAgent = Union[thgrepo.RepoAgent, filedialogs._UnfilteredRepoAgentProxy]
+
 mqpatchmimetype = 'application/thg-mqunappliedpatch'
 
 # pick names from "hg help templating" if any
@@ -98,11 +119,13 @@ ALLCOLUMNS = tuple(name for name, _text in COLUMNHEADERS)
 UNAPPLIED_PATCH_COLOR = QColor('#999999')
 HIDDENREV_COLOR = QColor('#666666')
 TROUBLED_COLOR = QColor(172, 34, 34)
+PUBLISHED_TOPIC_COLOR = QColor('#666666')
 
-GraphNodeRole = Qt.UserRole + 0
-LabelsRole = Qt.UserRole + 1  # [(text, style), ...]
+GraphNodeRole = Qt.ItemDataRole.UserRole + 0
+LabelsRole = Qt.ItemDataRole.UserRole + 1  # [(text, style), ...]
 
 def _parsebranchcolors(value):
+    # type: (Text) -> List[Tuple[Text, Text]]
     r"""Parse tortoisehg.branchcolors setting
 
     >>> _parsebranchcolors('foo:#123456  bar:#789abc ')
@@ -160,12 +183,13 @@ class HgRepoListModel(QAbstractTableModel):
     _mqtags = (b'qbase', b'qtip', b'qparent')
 
     def __init__(self, repoagent, parent=None):
+        # type: (RepoAgent, Optional[QObject]) -> None
         QAbstractTableModel.__init__(self, parent)
         self._cache = []
         self._timerhandle = None
         self._rowcount = 0
         self._repoagent = repoagent
-        self._selectedrevs = frozenset([])
+        self._selectedrevs = frozenset([])  # type: FrozenSet[int]
         self._revspec = ''
         self._filterbyrevset = True
         self.unicodestar = True
@@ -196,6 +220,7 @@ class HgRepoListModel(QAbstractTableModel):
         return self._repoagent.rawRepo()
 
     def _initBranchColors(self):
+        # type: () -> None
         # Always assign the first color to the default branch
         self._branch_colors['default'] = graph.COLORS[0]
 
@@ -204,6 +229,7 @@ class HgRepoListModel(QAbstractTableModel):
             self._repoagent.configString('tortoisehg', 'branchcolors')))
 
     def setBranch(self, branch, allparents=False):
+        # type: (Text, bool) -> None
         branchchanged = (branch != self._filterbranch)
         parentchanged = (allparents != self._allparents)
         self._filterbranch = branch
@@ -212,6 +238,7 @@ class HgRepoListModel(QAbstractTableModel):
             self._rebuildGraph()
 
     def setShowGraftSource(self, visible):
+        # type: (bool) -> None
         if self._showgraftsource == visible:
             return
         self._showgraftsource = visible
@@ -245,6 +272,7 @@ class HgRepoListModel(QAbstractTableModel):
 
     @pyqtSlot()
     def _reloadGraph(self):
+        # type: () -> None
         self._branchheads.clear()
         self._latesttags = {-1: self._latesttags[-1]}  # clear
         if self._revspec:
@@ -252,6 +280,7 @@ class HgRepoListModel(QAbstractTableModel):
         self._rebuildGraph()
 
     def _rebuildGraph(self):
+        # type: () -> None
         if not self._querysess.isFinished():
             self._pendingrebuild = True
             return
@@ -320,9 +349,11 @@ class HgRepoListModel(QAbstractTableModel):
         self.revsUpdated.emit()
 
     def revset(self):
+        # type: () -> Text
         return self._revspec
 
     def setRevset(self, revspec):
+        # type: (Text) -> None
         revspec = pycompat.unicode(revspec)
         if revspec == self._revspec:
             return
@@ -334,6 +365,7 @@ class HgRepoListModel(QAbstractTableModel):
         self._runQuery()
 
     def _runQuery(self):
+        # type: () -> None
         self.showMessage.emit(_('Searching...'))
 
         self._querysess.abort()
@@ -344,6 +376,7 @@ class HgRepoListModel(QAbstractTableModel):
 
     @pyqtSlot(int)
     def _onQueryFinished(self, ret):
+        # type: (int) -> None
         sess = self._querysess
         if not sess.isFinished():
             # new query is already running
@@ -361,6 +394,7 @@ class HgRepoListModel(QAbstractTableModel):
             self._rebuildGraph()  # invalid revspec
 
     def _applyRevsetResult(self, revset):
+        # type: (List[int]) -> None
         self._selectedrevs = frozenset(revset)
         if self._filterbyrevset or self._pendingrebuild:
             self._rebuildGraph()
@@ -369,6 +403,7 @@ class HgRepoListModel(QAbstractTableModel):
             self.revsUpdated.emit()  # some revisions may be disabled
 
     def setFilterByRevset(self, filtered):
+        # type: (bool) -> None
         if self._filterbyrevset == filtered:
             return
         self._filterbyrevset = filtered
@@ -376,6 +411,7 @@ class HgRepoListModel(QAbstractTableModel):
             self._rebuildGraph()
 
     def _reloadConfig(self):
+        # type: () -> None
         self._fill_step = self._repoagent.configInt('tortoisehg', 'graphlimit')
         self._authorcolor = self._repoagent.configBool(
             'tortoisehg', 'authorcolor')
@@ -386,11 +422,13 @@ class HgRepoListModel(QAbstractTableModel):
 
     @pyqtSlot()
     def _invalidate(self):
+        # type: () -> None
         self._reloadConfig()
         self._cache = []
         self._emitAllDataChanged()
 
     def _emitAllDataChanged(self):
+        # type: () -> None
         if self._rowcount <= 0:
             return
         # optimize range if necessary
@@ -398,20 +436,24 @@ class HgRepoListModel(QAbstractTableModel):
         self.dataChanged.emit(self.index(0, 0), bottomright)
 
     def branch(self):
+        # type: () -> Text
         return self._filterbranch
 
     def canFetchMore(self, parent):
+        # type: (QModelIndex) -> bool
         if parent.isValid():
             return False
         return not self.graph.isfilled()
 
     def fetchMore(self, parent):
+        # type: (QModelIndex) -> None
         if parent.isValid() or self.graph.isfilled():
             return
         self.graph.build_nodes(self._fill_step)
         self._expandRowCount()
 
     def _ensureBuilt(self, rev):
+        # type: (Optional[int]) -> None
         """
         Make sure rev data is available (graph element created).
 
@@ -422,6 +464,7 @@ class HgRepoListModel(QAbstractTableModel):
         # caller should do _expandRowCount() or _shrinkRowCount() by itself
 
     def loadall(self):
+        # type: () -> None
         self._timerhandle = self.startTimer(1)
 
     def timerEvent(self, event):
@@ -438,6 +481,7 @@ class HgRepoListModel(QAbstractTableModel):
                 self.showMessage.emit('')
 
     def _expandRowCount(self):
+        # type: () -> None
         newlen = len(self.graph)
         if newlen > self._rowcount:
             self.beginInsertRows(QModelIndex(), self._rowcount, newlen - 1)
@@ -445,6 +489,7 @@ class HgRepoListModel(QAbstractTableModel):
             self.endInsertRows()
 
     def _shrinkRowCount(self):
+        # type: () -> None
         newlen = len(self.graph)
         if newlen < self._rowcount:
             self.beginRemoveRows(QModelIndex(), newlen, self._rowcount - 1)
@@ -452,22 +497,25 @@ class HgRepoListModel(QAbstractTableModel):
             self.endRemoveRows()
 
     def rowCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         if parent.isValid():
             return 0
         return self._rowcount
 
     def columnCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         if parent.isValid():
             return 0
         return len(self.allColumns())
 
     def maxWidthValueForColumn(self, column):
+        # type: (int) -> Optional[Text]
         if column == RevColumn:
             return '8' * len(str(len(self.repo))) + '+'
         if column in (NodeColumn, GitNodeColumn):
             return '8' * 12 + '+'
         if column in (LocalDateColumn, UtcDateColumn):
-            return hglib.displaytime(dateutil.makedate())
+            return hglib.tounicode(hglib.displaytime(dateutil.makedate()))
         if column in (TagsColumn, LatestTagColumn):
             try:
                 return sorted(self.repo.tags().keys(), key=lambda x: len(x))[-1][:10]
@@ -486,6 +534,7 @@ class HgRepoListModel(QAbstractTableModel):
         return None
 
     def rev(self, index):
+        # type: (QModelIndex) -> int
         """Revision number of the specified row; None for working-dir"""
         if not index.isValid():
             return -1
@@ -496,6 +545,7 @@ class HgRepoListModel(QAbstractTableModel):
         return gnode.rev
 
     def _user_color(self, user):
+        # type: (Text) -> Text
         'deprecated, please replace with hgtk color scheme'
         if user not in self._user_colors:
             idx = graph.hashcolor(user)
@@ -503,27 +553,29 @@ class HgRepoListModel(QAbstractTableModel):
         return self._user_colors[user]
 
     def _namedbranch_color(self, branch):
+        # type: (Text) -> Text
         'deprecated, please replace with hgtk color scheme'
         if branch not in self._branch_colors:
             idx = graph.hashcolor(branch)
             self._branch_colors[branch] = graph.COLORS[idx]
         return self._branch_colors[branch]
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
+        # type: (QModelIndex, int) -> Any
         if not index.isValid():
             return None
         gnode = self.graph[index.row()]
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             if self._hasFileColumn() and index.column() == FileColumn:
                 return hglib.tounicode(gnode.extra[0])
-        if role == Qt.FontRole:
+        if role == Qt.ItemDataRole.FontRole:
             if index.column() in (NodeColumn, GitNodeColumn, ConvertedColumn):
                 return QFont("Monospace")
             if index.column() == DescColumn and gnode.wdparent:
                 font = QApplication.font('QAbstractItemView')
                 font.setBold(True)
                 return font
-        if role == Qt.ForegroundRole:
+        if role == Qt.ItemDataRole.ForegroundRole:
             if (gnode.shape == graph.NODE_SHAPE_UNAPPLIEDPATCH
                 and index.column() != DescColumn):
                 return UNAPPLIED_PATCH_COLOR
@@ -536,12 +588,13 @@ class HgRepoListModel(QAbstractTableModel):
         except error.RevlogError as e:
             if 'THGDEBUG' in os.environ:
                 raise
-            if role == Qt.DisplayRole:
+            if role == Qt.ItemDataRole.DisplayRole:
                 return hglib.tounicode(str(e))
             else:
                 return None
 
     def _safedata(self, index, role):
+        # type: (QModelIndex, int) -> Any
         row = index.row()
         graphlen = len(self.graph)
         cachelen = len(self._cache)
@@ -563,20 +616,18 @@ class HgRepoListModel(QAbstractTableModel):
         return data[idx]
 
     def _rawdata(self, index, role):
+        # type: (QModelIndex, int) -> Any
         row = index.row()
         column = index.column()
         gnode = self.graph[row]
         ctx = self.repo[gnode.rev]
 
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             textfunc = self._columnmap.get(column)
             if textfunc is None:
                 return None
-            text = textfunc(self, ctx)
-            if not isinstance(text, pycompat.unicode):
-                text = hglib.tounicode(text)
-            return text
-        elif role == Qt.ForegroundRole:
+            return textfunc(self, ctx)
+        elif role == Qt.ItemDataRole.ForegroundRole:
             color = None
             if gnode.instabilities:
                 color = TROUBLED_COLOR
@@ -591,6 +642,11 @@ class HgRepoListModel(QAbstractTableModel):
                         color = HIDDENREV_COLOR
                     else:
                         color = color.lighter()
+            if column == TopicColumn and not ctx.mutable():
+                if color is None:
+                    color = PUBLISHED_TOPIC_COLOR
+                else:
+                    color = color.lighter()
             return color
         elif role == LabelsRole and column == DescColumn:
             return self._getrevlabels(ctx)
@@ -599,23 +655,25 @@ class HgRepoListModel(QAbstractTableModel):
         return None
 
     def flags(self, index):
+        # type: (QModelIndex) -> int
         flags = super(HgRepoListModel, self).flags(index)
         if not index.isValid():
             return flags
         row = index.row()
         if row >= len(self.graph) and not self.repo.ui.debugflag:
             # TODO: should not happen; internal data went wrong (issue #754)
-            return Qt.NoItemFlags
+            return Qt.ItemFlag.NoItemFlags
         rev, isunapplied = self.graph.getrevstate(row)
         if not self.isActiveRev(rev):
-            return Qt.NoItemFlags
+            return Qt.ItemFlag.NoItemFlags
         if isunapplied:
-            flags |= Qt.ItemIsDragEnabled | Qt.ItemIsDropEnabled
+            flags |= Qt.ItemFlag.ItemIsDragEnabled | Qt.ItemFlag.ItemIsDropEnabled
         if rev is None:
-            flags |= Qt.ItemIsDropEnabled
+            flags |= Qt.ItemFlag.ItemIsDropEnabled
         return flags
 
     def isActiveRev(self, rev):
+        # type: (int) -> bool
         """True if the specified rev is not excluded by revset"""
         return (not self._revspec
                 or rev in self._selectedrevs
@@ -624,12 +682,14 @@ class HgRepoListModel(QAbstractTableModel):
                     and not self._querysess.isFinished()))
 
     def mimeTypes(self):
+        # type: () -> List[Text]
         return [mqpatchmimetype]
 
     def supportedDropActions(self):
-        return Qt.MoveAction
+        return Qt.DropAction.MoveAction
 
     def mimeData(self, indexes):
+        # type: (List[QModelIndex]) -> QMimeData
         data = set()
         for index in indexes:
             row = b'%d' % index.row()
@@ -641,6 +701,7 @@ class HgRepoListModel(QAbstractTableModel):
         return qmd
 
     def dropMimeData(self, data, action, row, column, parent):
+        # type: (QMimeData, int, int, int, QModelIndex) -> bool
         if mqpatchmimetype not in data.formats():
             return False
         dragrows = [int(r)
@@ -664,11 +725,13 @@ class HgRepoListModel(QAbstractTableModel):
         self._repoagent.runCommand(cmdline)
         return True
 
-    def headerData(self, section, orientation, role=Qt.DisplayRole):
-        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
+    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
+        # type: (int, int, int) -> Optional[Text]
+        if orientation == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.DisplayRole:
             return self.allColumnHeaders()[section][1]
 
     def defaultIndex(self):
+        # type: () -> QModelIndex
         """Index that should be selected when the model is initially loaded
         or the row previously selected is gone"""
         repo = self.repo
@@ -678,11 +741,11 @@ class HgRepoListModel(QAbstractTableModel):
                     'tip': b'tip',
                     'workingdir': None,
                     }.get(initialsel, b'.')
-        rev = repo[changeid].rev()
+        rev = repo[changeid].rev()  # type: int
         if self._selectedrevs and rev not in self._selectedrevs:
             rev = max(self._selectedrevs)
         index = self.indexFromRev(rev)
-        if index.flags() & Qt.ItemIsEnabled:
+        if index.flags() & Qt.ItemFlag.ItemIsEnabled:
             return index
 
         if self._filterbranch:
@@ -693,11 +756,12 @@ class HgRepoListModel(QAbstractTableModel):
                 if not isinstance(gnode.rev, int):
                     continue
                 index = self.index(row, 0)
-                if index.flags() & Qt.ItemIsEnabled:
+                if index.flags() & Qt.ItemFlag.ItemIsEnabled:
                     return index
         return QModelIndex()
 
     def indexFromRev(self, rev):
+        # type: (int) -> QModelIndex
         self._ensureBuilt(rev)
         self._expandRowCount()
         try:
@@ -707,6 +771,7 @@ class HgRepoListModel(QAbstractTableModel):
         return self.index(row, 0)
 
     def _getbranch(self, ctx):
+        # type: (context.changectx) -> Text
         b = hglib.tounicode(ctx.branch())
         assert b is not None
         if ctx.extra().get(b'close'):
@@ -717,6 +782,7 @@ class HgRepoListModel(QAbstractTableModel):
         return b
 
     def _getlatesttags(self, ctx):
+        # type: (context.changectx) -> Text
         rev = ctx.rev()
         todo = [rev]
         repo = self.repo
@@ -747,12 +813,14 @@ class HgRepoListModel(QAbstractTableModel):
         return self._latesttags[rev][2]
 
     def _gettags(self, ctx):
+        # type: (context.changectx) -> Text
         if ctx.rev() is None:
             return ''
         tags = [hglib.tounicode(t) for t in ctx.tags() if t not in self._mqtags]
         return ','.join(tags)
 
     def _getrev(self, ctx):
+        # type: (context.changectx) -> Text
         rev = ctx.rev()
         if isinstance(rev, int):
             return str(rev)
@@ -762,15 +830,17 @@ class HgRepoListModel(QAbstractTableModel):
             return ''
 
     def _getauthor(self, ctx):
+        # type: (context.changectx) -> Text
         try:
             user = ctx.user()
             if not self._fullauthorname:
                 user = hglib.username(user)
-            return user
+            return hglib.tounicode(user)
         except error.Abort:
             return _('Mercurial User')
 
     def _getlog(self, ctx):
+        # type: (context.changectx) -> Text
         if ctx.rev() is None:
             if self.unicodestar:
                 # The Unicode symbol is a black star:
@@ -784,6 +854,7 @@ class HgRepoListModel(QAbstractTableModel):
         return hglib.longsummary(ctx.description(), limit)
 
     def _getrevlabels(self, ctx):
+        # type: (context.changectx) -> List[Tuple[Text, Text]]
         labels = []
 
         # as of hg 4.4.2, repo.branchheads() can be slow because of
@@ -807,9 +878,10 @@ class HgRepoListModel(QAbstractTableModel):
             if ctx.node() in branchheads:
                 labels.append((hglib.tounicode(branch), 'log.branch'))
 
-        if ctx.thgmqunappliedpatch():
+        # ctx is really thgrepo.thgchangectx, but not accessible here
+        if ctx.thgmqunappliedpatch():  # pytype: disable=attribute-error
             style = 'log.unapplied_patch'
-            labels.append((hglib.tounicode(ctx._patchname), style))
+            labels.append((hglib.tounicode(ctx._patchname), style))  # pytype: disable=attribute-error
 
         for mark in ctx.bookmarks():
             style = 'log.bookmark'
@@ -819,7 +891,7 @@ class HgRepoListModel(QAbstractTableModel):
                     style = 'log.curbookmark'
             labels.append((hglib.tounicode(mark), style))
 
-        for tag in ctx.thgtags():
+        for tag in ctx.thgtags():  # pytype: disable=attribute-error
             if self.repo.thgmqtag(tag):
                 style = 'log.patch'
             else:
@@ -840,9 +912,12 @@ class HgRepoListModel(QAbstractTableModel):
         return labels
 
     def _getchanges(self, ctx):
+        # type: (context.changectx) -> List[Tuple[Text, Text]]
         """Return the MAR status for the given ctx."""
         labels = []
-        M, A, R = ctx.changesToParent(0)
+
+        # ctx is really thgrepo.thgchangectx, but not accessible here
+        M, A, R = ctx.changesToParent(0)  # pytype: disable=attribute-error
         if A:
             labels.append((str(len(A)), 'log.added'))
         if M:
@@ -852,6 +927,7 @@ class HgRepoListModel(QAbstractTableModel):
         return labels
 
     def _getconv(self, ctx):
+        # type: (context.changectx) -> Text
         if ctx.rev() is not None:
             extra = ctx.extra()
             cvt = extra.get(b'convert_revision', b'')
@@ -870,20 +946,24 @@ class HgRepoListModel(QAbstractTableModel):
         return ''
 
     def _getphase(self, ctx):
+        # type: (context.changectx) -> Text
         if ctx.rev() is None:
             return ''
         try:
-            return ctx.phasestr()
+            return hglib.tounicode(ctx.phasestr())
         except:
             return 'draft'
 
     def _gettopic(self, ctx):
+        # type: (context.changectx) -> Optional[Text]
         return hglib.tounicode(ctx.extra().get(b'topic'))
 
     def _hasFileColumn(self):
+        # type: () -> bool
         return False # no FileColumn
 
     def allColumns(self):
+        # type: () -> Tuple[Text, ...]
         if self._hasFileColumn():
             return ALLCOLUMNS
         else:
@@ -905,7 +985,7 @@ class HgRepoListModel(QAbstractTableModel):
         NodeColumn: lambda self, ctx: str(ctx),
         GitNodeColumn: lambda self, ctx: hglib.gitcommit_short(ctx),
         AgeColumn: lambda self, ctx: hglib.age(ctx.date()).decode('utf-8'),
-        LocalDateColumn: lambda self, ctx: hglib.displaytime(ctx.date()),
+        LocalDateColumn: lambda self, ctx: hglib.displaytime(ctx.date()).decode('utf-8'),
         UtcDateColumn: lambda self, ctx: hglib.utctime(ctx.date()),
         ConvertedColumn: _getconv,
         PhaseColumn: _getphase,
@@ -923,6 +1003,7 @@ class FileRevModel(HgRepoListModel):
                        'Age', 'Filename')
 
     def __init__(self, repoagent, filename, parent=None):
+        # type: (RepoAgent, bytes, Optional[QObject]) -> None
         self._filename = filename
         HgRepoListModel.__init__(self, repoagent, parent)
 
@@ -934,6 +1015,7 @@ class FileRevModel(HgRepoListModel):
         return graph.Graph(self.repo, grapher)
 
     def indexLinkedFromRev(self, rev):
+        # type: (Optional[int]) -> QModelIndex
         """Index for the last changed revision before the specified revision
 
         This does not follow renames.
@@ -949,6 +1031,7 @@ class FileRevModel(HgRepoListModel):
         return self.indexFromRev(fctx.linkrev())
 
     def fileData(self, index, baseindex=QModelIndex()):
+        # type: (QModelIndex, QModelIndex) -> Union[filedata.FileData, filedata.PatchFileData]
         """Displayable file data at the given index; baseindex specifies the
         revision where status is calculated from"""
         row = index.row()
