@@ -93,7 +93,7 @@ def run():
         # TODO: Figure out if this hack for py2exe is still needed, and possibly
         #       wrap this to handle unicode on py3
         if not pycompat.ispy3:
-            mystderr = pycompat.bytesio()
+            mystderr = util.bytesio()
             sys.stderr = mystderr
             sys.__stdout__ = sys.stdout
             sys.__stderr__ = sys.stderr
